@@ -25,9 +25,9 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
             command_data = self._encode_int16(command_data)
 
         LEN = len(command_data) + 1
-        check_digit = sum(command_data[1:]) + genre
-        if check_digit >= 256:
-            check_digit %= 256
+        check_digit = (sum(command_data[1:]) + genre) & 0xff
+        # if check_digit >= 256:
+        #     check_digit %= 256
         command = [
             ProtocolCode.HEADER,
             ProtocolCode.HEADER,
@@ -129,14 +129,26 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
             id: 0/1/2/3 (ALL/L/R/W)
         """
         self._mesg(ProtocolCode.READ_NEXT_ERROR, id, has_reply=True)
+        
+    def set_fresh_mode(self, id, mode):
+        """Set command refresh mode
+        
+        Args:
+            id: 1/2 (L/R).\n
+            mode: int
+                0 - Always execute the latest command first.
+                1 - Execute instructions sequentially in the form of a queue. 
+        """
+        return self._mesg(ProtocolCode.SET_FRESH_MODE, id, mode)
 
-    def set_free_mode(self, id):
+    def set_free_mode(self, id, value):
         """set free mode
 
         Args:
             id: 0/1/2/3 (ALL/L/R/W)
+            value: 0 - close 1 - open
         """
-        return self._mesg(ProtocolCode.SET_FREE_MODE, id)
+        return self._mesg(ProtocolCode.SET_FREE_MODE, id, value)
 
     def is_free_mode(self, id):
         """Check if it is free mode
@@ -155,7 +167,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         """ Get the degree of all joints.
 
         Args:
-            id: 1/2/3 (L/R/W)
+            id: 1/2 (L/R)
             
         Return:
             list: A float list of all degree.
@@ -170,29 +182,30 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
             joint: 1 ~ 6
             angle: int
             speed: 1 ~ 100
+        
+        Return:
+            None
         """
         return self._mesg(
             ProtocolCode.SEND_ANGLE, id, joint, [self._angle2int(angle)], speed
         )
 
-    def send_angles(self, id, degrees, speed, mode):
+    def send_angles(self, id, degrees, speed):
         """Send all angles to the robotic arm
 
         Args:
-            id: 1/2/3 (L/R/W).
-            degrees: [angle_list]
+            id: 1/2 (L/R).
+            degrees: [angle_list] len 6
             speed: 1 - 100
-            mode: 0 - with interpolation 1 - No interpolation 2 - reserved.
-
         """
         degrees = [self._angle2int(degree) for degree in degrees]
-        return self._mesg(ProtocolCode.SEND_ANGLES, id, degrees, speed, mode)
+        return self._mesg(ProtocolCode.SEND_ANGLES, id, degrees, speed)
 
     def get_coords(self, id):
         """Get the coordinates of the robotic arm
         
         Args:
-            id: 1/2/3 (L/R/W).
+            id: 1/2 (L/R).
         """
         return self._mesg(ProtocolCode.GET_COORDS, id, has_reply=True)
 
@@ -202,7 +215,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         Args:
             id: 1/2/3 (L/R/W).
             coord: 1 ~ 6 (x/y/z/rx/ry/rz)
-            data: int
+            data: Coordinate value
             speed: 0 ~ 100
         """
         value = self._coord2int(data) if coord <= 3 else self._angle2int(data)
@@ -212,7 +225,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         """Send all coords to robot arm.
 
         Args:
-            id: 1/2/3 (L/R/W).
+            id: 1/2 (L/R).
             coords: a list of coords value(List[float]), length 6, [x(mm), y, z, rx(angle), ry, rz]\n
             speed : (int) 0 ~ 100
             mode : (int) 0 - moveJ, 1 - moveL, 2 - moveC
@@ -224,6 +237,24 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         for angle in coords[3:]:
             coord_list.append(self._angle2int(angle))
         return self._mesg(ProtocolCode.SEND_COORDS, id, coord_list, speed, mode)
+    
+    def get_angle(self, id, joint_id):
+        """Get the angle of a single joint
+        
+        Args:
+            id (int): 1/2/3 (L/R/W).
+            joint_id (int): 1 - 7 (7 is gripper)
+        """
+        return self._mesg(ProtocolCode.GET_ANGLE, id, joint_id, has_reply = True)
+    
+    def get_coord(self, id, joint_id):
+        """Read a single coordinate parameter
+        
+        Args:
+            id (int): 1/2/3 (L/R/W).
+            joint_id (int): 1 - 7 (7 is gripper)
+        """
+        return self._mesg(ProtocolCode.GET_COORD, id, joint_id, has_reply = True)
 
     def pause(self, id):
         """Pause movement
@@ -262,13 +293,14 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         """
         return self._mesg(ProtocolCode.STOP, id)
 
-    def is_in_position(self, id, mode, data):  # TODO 通信协议可能有问题，待完善
+    def is_in_position(self, id, data, mode):  # TODO 通信协议可能有问题，待完善
         """Judge whether in the position.
 
         Args:
             id: 0/1/2/3 (ALL/L/R/W).
+            data: A data list, angles or coords. If id is 1/2. data length is 6. If id is 0. data len 13. if id is 3. data len 1
             mode: 1 - coords, 0 - angles
-            data: A data list, angles or coords, length 6.
+            
 
         Return:
             1 - True
@@ -276,14 +308,14 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
             -1 - Error
         """
         if mode == 1:
-            self.calibration_parameters(coords=data)
+            # self.calibration_parameters(coords=data)
             data_list = []
             for idx in range(3):
                 data_list.append(self._coord2int(data[idx]))
             for idx in range(3, 6):
                 data_list.append(self._angle2int(data[idx]))
         elif mode == 0:
-            self.calibration_parameters(degrees=data)
+            # self.calibration_parameters(degrees=data)
             data_list = [self._angle2int(i) for i in data]
         else:
             raise Exception("id is not right, please input 0 or 1")
@@ -344,7 +376,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         Args:
             id: 1/2/3 (L/R/W).
             joint_id: int 1-6.
-            increment: # TODO 未注明
+            increment: 
             speed: int (1 - 100)
         """
         return self._mesg(ProtocolCode.JOG_INCREMENT, id, joint_id, increment, speed)
@@ -379,16 +411,15 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         """
         return self._mesg(ProtocolCode.GET_ENCODER, id, joint_id, has_reply=True)
 
-    def set_encoders(self, id, encoders, speed, mode):
+    def set_encoders(self, id, encoders, speed):
         """Set the six joints of the manipulator to execute synchronously to the specified position.
 
         Args:
-            id: 1/2/3 (L/R/W).
+            id: 1/2 (L/R).
             encoders: A encoder list, length 6.
             speed: speed 1 ~ 100
-            mode: 0 - with interpolation 1 - No interpolation 2 - reserved.
         """
-        return self._mesg(ProtocolCode.SET_ENCODERS, id, encoders, speed, mode)
+        return self._mesg(ProtocolCode.SET_ENCODERS, id, encoders, speed)
 
     def get_encoders(self, id):
         """Get the six joints of the manipulator
@@ -419,7 +450,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
             id: 1/2/3 (L/R/W)
             speed (int): 0 - 100
         """
-        self.calibration_parameters(speed=speed)
+        # self.calibration_parameters(speed=speed)
         return self._mesg(ProtocolCode.SET_SPEED, id, speed)
 
     def get_acceleration(self, id):
@@ -449,7 +480,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         Returns:
             angle value(float)
         """
-        self.calibration_parameters(id=joint_id)
+        # self.calibration_parameters(id=joint_id)
         return self._mesg(
             ProtocolCode.GET_JOINT_MIN_ANGLE, id, joint_id, has_reply=True
         )
@@ -464,7 +495,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         Return:
             angle value(float)
         """
-        self.calibration_parameters(id=joint_id)
+        # self.calibration_parameters(id=joint_id)
         return self._mesg(
             ProtocolCode.GET_JOINT_MAX_ANGLE, id, joint_id, has_reply=True
         )
@@ -623,7 +654,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
             frequency (int): clock frequency (0/1: 0 - 1Mhz 1 - 10Mhz)
             pin_val (int): Duty cycle 0 ~ 100: 0 ~ 100%
         """
-        return self._mesg(ProtocolCode.SET_PWM_OUTPUT, channel, [frequency], pin_val)
+        return self._mesg(ProtocolCode.SET_PWM_OUTPUT, id, channel, [frequency], pin_val)
 
     def get_gripper_value(self, id):
         """Get the value of gripper.
@@ -645,12 +676,13 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         """
         return self._mesg(ProtocolCode.SET_GRIPPER_STATE, id, flag)
 
-    def set_gripper_value(self, id, value):
+    def set_gripper_value(self, id, value, speed):
         """Set gripper value
 
         Args:
             id: 1/2 (L/R)
             value (int): 0 ~ 100
+            speed (int): 0 - 100
         """
         return self._mesg(ProtocolCode.SET_GRIPPER_VALUE, id, value)
 
@@ -695,7 +727,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
             id: 0/1/2 (ALL/L/R)
             coords: a list of coords value(List[float]), length 6. [x(mm), y, z, rx(angle), ry, rz]
         """
-        self.calibration_parameters(coords=coords)
+        # self.calibration_parameters(coords=coords)
         coord_list = []
         for idx in range(3):
             coord_list.append(self._coord2int(coords[idx]))
@@ -718,7 +750,7 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
             id: 0/1/2 (ALL/L/R)
             coords: a list of coords value(List[float]), length 6 [x(mm), y, z, rx(angle), ry, rz]
         """
-        self.calibration_parameters(coords=coords)
+        # self.calibration_parameters(coords=coords)
         coord_list = []
         for idx in range(3):
             coord_list.append(self._coord2int(coords[idx]))
@@ -815,33 +847,6 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         """
         return self._mesg(ProtocolCode.GET_JOINT_CURRENT, id, joint_id)
 
-    # Basic
-    def set_basic_mode(self, pin_no, pin_mode):
-        """Set base IO, input or output mode
-
-        Args:
-            pin_no: 1 - 5
-            pin_mode: 0 - input 1 - output
-        """
-        return self._mesg(ProtocolCode.SET_BASIC_OUTPUT, 0, pin_no, pin_mode)
-
-    def set_basic_output(self, pin_no, pin_signal):
-        """Set basic output.
-
-        Args:
-            pin_no: pin port number (0 - 20).
-            pin_signal: 0 / 1 (0 - low, 1 - high)
-        """
-        return self._mesg(ProtocolCode.GET_BASIC_INPUT, 0, pin_no, pin_signal)
-
-    def get_basic_input(self, pin_no):
-        """Get basic input for M5 version.
-
-        Args:
-            pin_no: (int) pin port number (0 - 20).
-        """
-        return self._mesg(ProtocolCode.GET_BASE_INPUT, 0, pin_no, has_reply=True)
-
     def get_plan_speed(self, id = 0):
         """Get planning speed
 
@@ -925,6 +930,117 @@ class MyBuddyCommandGenerator(MyCobotCommandGenerator):
         """
         return self._mesg(0xE6, id, has_reply=True)
     
+    def get_base_coords(self, *args: int):
+        """Convert coordinates to base coordinates. Pass in parameters or no parameters
+        
+        Args:
+            coords: a list of coords value(List[float]), length 6 [x(mm), y, z, rx(angle), ry, rz]
+            arm: 0 - left. 1 - right
+            
+        Return:
+            Base coords
+        """
+        if len(args) == 2:
+            coords, arm = args
+            coord_list = []
+            for idx in range(3):
+                coord_list.append(self._coord2int(coords[idx]))
+            for angle in coords[3:]:
+                coord_list.append(self._angle2int(angle))
+            return self._mesg(ProtocolCode.GET_BASE_COORDS, 0, coord_list, arm, has_reply = True)
+        elif len(args) == 0:
+            return self._mesg(ProtocolCode.GET_ALL_BASE_COORDS, 0, has_reply = True)
+    
+    def base_to_single_coords(self, base_coords, arm):
+        """Convert base coordinates to coordinates
+        
+        Args:
+            coords: a list of base coords value len 6
+            arm: 0 - left. 1 - right
+            
+        Return:
+            coords
+        """
+        coord_list = []
+        for idx in range(3):
+            coord_list.append(self._coord2int(base_coords[idx]))
+        for angle in base_coords[3:]:
+            coord_list.append(self._angle2int(angle))
+        return self._mesg(ProtocolCode.BASE_TO_SINGLE_COORDS, 0, coord_list, arm, has_reply = True)
+    
+    def collision(self, left_angles, right_angles):
+        """Collision detection main program
+        
+        Args:
+            left_angles: left arm angle len 6.
+            right_angles: right arm angle len 6.
+            
+        Return:
+            int
+        """
+        degrees1 = [self._angle2int(degree) for degree in left_angles]
+        degrees2 = [self._angle2int(degree) for degree in right_angles]
+        
+        return self._mesg(ProtocolCode.COLLISION, 0, degrees1, degrees2, has_reply = True)
+    
+    def get_base_coord(self, id):
+        """Get the base coordinates of the single arm
+        
+        Args:
+            id: 1/2 (L/R)
+        """
+        return self._mesg(ProtocolCode.GET_BASE_COORD, id, has_reply = True)
+    
+    def write_base_coord(self, id, axis, coord, speed):
+        """Base single coordinate movement
+        
+        Args:
+            id: 1/2 (L/R)
+            axis: 1 - 6 (x/y/z/rx/ry/rz)
+            coord: Coordinate value
+            speed: 1 - 100
+        """
+        value = self._coord2int(coord) if axis <= 3 else self._angle2int(coord)
+        return self._mesg(ProtocolCode.WRITE_BASE_COORD, id, axis, [value], speed)
+    
+    def write_base_coords(self, id, coords, speed):
+        """base coordinate move
+        
+        Args:
+            id: 1/2 (L/R)
+            coords: coords: a list of coords value(List[float]), length 6, [x(mm), y, z, rx(angle), ry, rz]
+            speed: 1 - 100
+        """
+        coord_list = []
+        for idx in range(3):
+            coord_list.append(self._coord2int(coords[idx]))
+        for angle in coords[3:]:
+            coord_list.append(self._angle2int(angle))
+        return self._mesg(ProtocolCode.WRITE_BASE_COORDS, id, coord_list, speed)
+    
+    def jog_inc_coord(self, axis, increment, speed):
+        """Double-arm coordinated coordinate stepping
+        
+        Args:
+            axis: 1 - 6 (x/y/z/rx/ry/rz)
+            increment: 
+            speed: 1 - 100
+        """
+        value = self._coord2int(increment) if axis <= 3 else self._angle2int(increment)
+        return self._mesg(ProtocolCode.JOG_INC_COORD, 0, axis, [value], speed)
+        
+    def collision_switch(self, state):
+        """Collision Detection Switch
+        
+        Args:
+            state (int): 0 - close 1 - open (Off by default)
+        """
+        return self._mesg(ProtocolCode.COLLISION_SWITCH, state)
+    
+    def is_collision_on(self):
+        """Get collision detection status"""
+        return self._mesg(ProtocolCode.IS_COLLISION_ON, 0, has_reply = True)
+        
     
     # def init_iic(self):
     #     from smbus2 import SMBus
